package org.zeroturnaround.javarebel.integration.spring;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.zeroturnaround.javarebel.ClassEventListener;
import org.zeroturnaround.javarebel.LoggerFactory;

/**
 * This is the central class of the JavaRebel Spring plugin. 
 * It is the entry point for both bean definition reloading ({@link #onClassEvent(int, Class)}) 
 * and bean reconfiguration ({@link #reloadBeanDefinitions()}.  
 * 
 * @author Jevgeni Kabanov
 */
public final class SpringBeanReloader implements ClassEventListener {  
  private static ThreadLocal changedBeans = new ThreadLocal();

  private ConfigurableListableBeanFactory bf;    

  public SpringBeanReloader(ConfigurableListableBeanFactory bf) {
    this.bf = bf;
  }

  /**
   * This method will reconfigure the singletons when their class changes. 
   * It will delay reconfiguration if we are running in a {@link #reloadBeanDefinitions()}
   * method to be sure we are reconfiguring against the latest definitions.
   */
  public void onClassEvent(int eventType, Class klass) {
    if (changedBeans.get() == null)
      reloadBeanDefinitions();

    String[] singletonNames = bf.getSingletonNames();

    for (int i = 0; i < singletonNames.length; i++) {
      final Object bean = bf.getBean(singletonNames[i]);
      final String singletonName = singletonNames[i];

      if (klass.isAssignableFrom(bean.getClass())) {
        Runnable run = new Runnable() {
          public void run() {
            LoggerFactory.getInstance().echo( 
                "JavaRebel-Spring: Reconfiguring bean '" + singletonName + "' [" + bean.getClass().getName() + "]");
            bf.configureBean(bean, singletonName);
          }
        };
        
        if (changedBeans.get() == null)
          run.run();
        else 
          ((List) changedBeans.get()).add(run);
      }
    }
  }

  /**
   * This method will reload the bean definitions and apply postprocessing. 
   * As it touches classes it may cause them to reload if they were changed.
   */
  public static void reloadBeanDefinitions() {
    changedBeans.set(new ArrayList());
    try {
      XmlBeanDefinitionManager.checkAndReload();
      AnnotationBeanDefinitionManager.rescan();
    }
    finally {
      List toRun = (List) changedBeans.get();
      changedBeans.set(null);
      
      for (Iterator i = toRun.iterator(); i.hasNext();) {
        Runnable run = (Runnable) i.next();
        run.run();
      }                 
    }
  }  
}